"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.callContract = void 0;

var _path = _interopRequireDefault(require("path"));

var _ethEnsNamehash = _interopRequireDefault(require("eth-ens-namehash"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _web = require("web3");

var _inquirer = require("inquirer");

var _cliUtils = require("./cli-utils");

var _helpers = require("./helpers");

const availableContracts = async abiDirPath => {
  const files = _fsExtra.default.readdirSync(abiDirPath).reduce((acc, file) => {
    if (file.endsWith('.json')) {
      acc.push(file.replace('.json', ''));
    }

    return acc;
  }, []);

  return files;
};

const selectContractName = async abiDirPath => {
  const contractNames = await availableContracts(abiDirPath);
  const answers = await (0, _inquirer.prompt)([{
    type: 'list',
    name: 'contract',
    message: 'Select contract: ',
    choices: contractNames
  }]);
  return answers.contract;
};

const getContractAddress = async (contractName, ethNetwork, config) => {
  if (config.contractAddresses && config.contractAddresses[contractName][ethNetwork]) {
    const confirmAnswers = await (0, _inquirer.prompt)([{
      type: 'confirm',
      name: 'confirmConfigAddress',
      message: `Create contract with address: ${config.contractAddresses[contractName][ethNetwork]}`
    }]);

    if (confirmAnswers.confirmConfigAddress) {
      return config.contractAddresses[contractName][ethNetwork];
    }
  }

  const answers = await (0, _inquirer.prompt)({
    type: 'input',
    name: 'address',
    message: 'Enter contract address'
  });
  return answers.address;
};

const selectContractMethod = async (web3Contract, contractName) => {
  const contractAbi = web3Contract._jsonInterface;
  const methods = contractAbi.reduce((acc, m) => {
    if (m.name && m.type === 'function') acc.push(m.name);
    return acc;
  }, []);

  _cliUtils.log.header(`${(0, _cliUtils.capitalize)(contractName)} Contract`);

  const answers = await (0, _inquirer.prompt)([{
    type: 'list',
    name: 'method',
    message: 'Select contract action: ',
    choices: methods
  }]);
  return answers.method;
};

const callContract = async (web3, ethNetwork, config = {}) => {
  const contractName = await selectContractName(config.abiDirPath);

  const abiFilePath = _path.default.resolve(config.abiDirPath, `${contractName}.json`);

  const abi = await _fsExtra.default.readJson(abiFilePath);
  const contractAddress = await getContractAddress(contractName, ethNetwork, config);
  const web3Contract = new web3.eth.Contract(abi, contractAddress);
  const method = await selectContractMethod(web3Contract, contractName);

  const abiMethod = web3Contract._jsonInterface.find(m => m.name === method);

  const questions = abiMethod.inputs.map(m => {
    const q = {
      type: 'input',
      name: m.name,
      message: `Enter ${m.name} (${m.type}): `,
      extraQuestions: [] // TODO: Ask extra questions inline
      // If uint, ask if it needs unit conversion

    };

    if (m.type.slice(0, 4) === 'uint') {
      q.extraQuestions = [{
        type: 'confirm',
        name: 'confirmUnitConvert',
        message: `Do you need to convert unit? `
      }, {
        type: 'list',
        name: 'conversion',
        message: `Conversion type: `,
        choices: ['fromWei', 'toWei']
      }, {
        type: 'input',
        name: 'decimals',
        message: `Enter conversion unit (e.g. 'ether' (1e18 wei), 'gwei' (1e9 wei)): `
      }];
    } else if (m.type === 'bytes32') {
      q.extraQuestions = [{
        type: 'confirm',
        name: 'confirmTextToData',
        message: `Do you need to convert text to data? `
      }, {
        type: 'list',
        name: 'encodeType',
        message: `Hash type: `,
        choices: ['nameHash', 'sha3', 'toHex']
      }];
    }

    return q;
  });
  const answers = await (0, _inquirer.prompt)(questions);
  const args = [];

  for (let i = 0; i < questions.length; i++) {
    const q = questions[i];
    let arg = answers[q.name];

    if (q.extraQuestions) {
      const extraAnswers = await (0, _inquirer.prompt)(q.extraQuestions.slice(0, 1));

      if (extraAnswers.confirmUnitConvert) {
        const convertAnswers = await (0, _inquirer.prompt)(q.extraQuestions.slice(1, 3));
        arg = _web.utils[convertAnswers.conversion](arg, convertAnswers.decimals);
      } else if (extraAnswers.confirmTextToData) {
        const dataAnswers = await (0, _inquirer.prompt)(q.extraQuestions.slice(1, 2));

        if (dataAnswers.encodeType === 'nameHash') {
          arg = _ethEnsNamehash.default.hash(arg);
        } else {
          arg = _web.utils[dataAnswers.encodeType](arg);
        }
      }
    }

    args.push(arg);
  }

  const readOnly = abiMethod.stateMutability === 'view' || abiMethod.constant === true;
  const methodType = readOnly ? 'call' : 'send';
  const account = await web3.eth.getCoinbase();
  const gasLimit = await (0, _helpers.estimateGas)(web3Contract, method, account, args);
  const gasPriceGwei = await (0, _helpers.getRecomendedGasPriceGwei)();

  const gasPrice = _web.utils.toWei(String(gasPriceGwei), 'gwei');

  const transactionOptions = {
    from: account,
    gasLimit,
    gasPrice
  };

  if (methodType === 'send') {
    const validateMessage = `
      Network: ${ethNetwork}

      Calling method: ${method}
      On contract: ${web3Contract._address}
      With args: ${args.toString()}
      From account: ${account}
      Gas Limit: ${transactionOptions.gasLimit}
      Gas Price: ${transactionOptions.gasPrice}\n
      Are your sure you'd like to proceed with this transaction?
    `;
    await validateTransaction(validateMessage);

    _cliUtils.log.info('Pending transaction...', 'blue');
  }

  const transaction = await web3Contract.methods[method](...args)[methodType](transactionOptions);

  _cliUtils.log.success('Transaction Complete!'); // eslint-disable-next-line no-console


  console.log('result:', transaction);
};

exports.callContract = callContract;

const validateTransaction = async message => {
  const answers = await (0, _inquirer.prompt)([{
    type: 'confirm',
    name: 'confirm',
    message: message
  }]);

  if (!answers.confirm) {
    throw new Error('Transaction cancelled');
  }
};